@@ -1,32 +1,9 @@
 // Protocol Buffers - Google's data interchange format
 // Copyright 2008 Google Inc.  All rights reserved.
-// https://developers.google.com/protocol-buffers/
 //
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions are
-// met:
-//
-//     * Redistributions of source code must retain the above copyright
-// notice, this list of conditions and the following disclaimer.
-//     * Redistributions in binary form must reproduce the above
-// copyright notice, this list of conditions and the following disclaimer
-// in the documentation and/or other materials provided with the
-// distribution.
-//     * Neither the name of Google Inc. nor the names of its
-// contributors may be used to endorse or promote products derived from
-// this software without specific prior written permission.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+// Use of this source code is governed by a BSD-style
+// license that can be found in the LICENSE file or at
+// https://developers.google.com/open-source/licenses/bsd
 
 package com.google.protobuf;
 
@@ -74,16 +51,10 @@ public final class MapLiteTest {
 
   @Test
   public void testSetMapValues() {
-    TestMap.Builder usingMutableMapBuilder = TestMap.newBuilder();
-    setMapValues(usingMutableMapBuilder);
-    TestMap usingMutableMap = usingMutableMapBuilder.build();
-    assertMapValuesSet(usingMutableMap);
-
-    TestMap.Builder usingAccessorsBuilder = TestMap.newBuilder();
-    setMapValues(usingAccessorsBuilder);
-    TestMap usingAccessors = usingAccessorsBuilder.build();
-    assertMapValuesSet(usingAccessors);
-    assertThat(usingMutableMap).isEqualTo(usingAccessors);
+    TestMap.Builder testMapBuilder = TestMap.newBuilder();
+    setMapValues(testMapBuilder);
+    TestMap testMap = testMapBuilder.build();
+    assertMapValuesSet(testMap);
   }
 
   private void copyMapValues(TestMap source, TestMap.Builder destination) {
@@ -587,6 +558,7 @@ public final class MapLiteTest {
   }
 
   @Test
+  @SuppressWarnings("SelfAssertion") // Just testing that the getters execute and are consistent.
   public void testGetMap() {
     TestMap.Builder builder = TestMap.newBuilder();
     setMapValues(builder);
